@@ -877,6 +877,7 @@ class SqlAlchemyTable(DBTable, abc.ABC):
     def __init__(self, *args, **kwargs):
         self.is_view = kwargs.get("is_view", False)
         self.lazy_fetch_primary_key = kwargs.get("lazy_fetch_primary_key", True)
+        self.name_in_db = kwargs.get("name_in_db")  # actual db table name, may differ from self.name
         super().__init__(*args, **kwargs)
 
     def __repr__(self):
@@ -896,11 +897,16 @@ class SqlAlchemyTable(DBTable, abc.ABC):
 
     #################### READ QUERY BUILDING & UTILS ####################
 
+    @property
+    def db_table_name(self) -> str:
+        # use name_in_db (actual db table name) if set, otherwise fall back to self.name
+        return self.name_in_db or self.name
+
     @property
     def _sa_table(self):
         with self.container.use_sa_engine() as sa_engine:
             return sa.Table(
-                self.name,
+                self.db_table_name,
                 self.container.sa_metadata,
                 autoload_with=sa_engine,
             )
@@ -1125,38 +1131,47 @@ class SqlAlchemyTable(DBTable, abc.ABC):
         fetch_chunk_size = fetch_chunk_size if fetch_chunk_size is not None else 100_000
         yield_chunk_size = yield_chunk_size if yield_chunk_size is not None else fetch_chunk_size
         stmt = self._sa_select(columns)
-        with self.container.use_sa_engine() as sa_engine:
-            chunk_idx = 0
-            total_time = 0
-            session = sessionmaker(bind=sa_engine)()
-            conn = session.connection()
-            results_generator = pd.read_sql_query(stmt, conn, dtype_backend="pyarrow", chunksize=fetch_chunk_size)
-            chunks_df = pd.DataFrame()
-            for chunk_df in results_generator:
-                t0 = time.time()
-                if where is not None:
-                    where_column, where_values = next(iter(where.items()))
-                    chunk_df = chunk_df[chunk_df[where_column].isin(where_values)]
-                if do_coerce_dtypes:
-                    chunk_df = coerce_dtypes_by_encoding(chunk_df, self.encoding_types)
-                # accumulate chunks
-                chunks_df = pd.concat([chunks_df, chunk_df], ignore_index=True)
-                if len(chunks_df) >= yield_chunk_size:
-                    # yield data once it reaches the yield_chunk_size
+        chunk_idx = 0
+        total_time = 0
+        # use context managers to ensure proper cleanup of engine and session
+        with (
+            self.container.use_sa_engine() as sa_engine,
+            sessionmaker(bind=sa_engine)() as session,
+        ):
+            results_generator = None
+            try:
+                conn = session.connection()
+                results_generator = pd.read_sql_query(stmt, conn, dtype_backend="pyarrow", chunksize=fetch_chunk_size)
+                chunks_df = pd.DataFrame()
+                for chunk_df in results_generator:
+                    t0 = time.time()
+                    if where is not None:
+                        where_column, where_values = next(iter(where.items()))
+                        chunk_df = chunk_df[chunk_df[where_column].isin(where_values)]
+                    if do_coerce_dtypes:
+                        chunk_df = coerce_dtypes_by_encoding(chunk_df, self.encoding_types)
+                    # accumulate chunks
+                    chunks_df = pd.concat([chunks_df, chunk_df], ignore_index=True)
+                    if len(chunks_df) >= yield_chunk_size:
+                        # yield data once it reaches the yield_chunk_size
+                        yield chunks_df
+                        chunks_df = pd.DataFrame()
+                    chunk_idx += 1
+                    total_time += time.time() - t0
+                    if chunk_idx % 10 == 0:
+                        _LOG.info(f"processed {chunk_idx} chunks in {total_time:.2f}s")
+                if len(chunks_df) > 0 or len(chunks_df.columns) > 0:
+                    # yield the remaining data
                     yield chunks_df
-                    chunks_df = pd.DataFrame()
-                chunk_idx += 1
-                total_time += time.time() - t0
-                if chunk_idx % 10 == 0:
-                    _LOG.info(f"processed {chunk_idx} chunks in {total_time:.2f}s")
-            if len(chunks_df) > 0 or len(chunks_df.columns) > 0:
-                # yield the remaining data
-                yield chunks_df
-            _LOG.info(f"finished reading {chunk_idx} chunks in {time.time() - t00:.2f}s (strategy=scan)")
+                _LOG.info(f"finished reading {chunk_idx} chunks in {time.time() - t00:.2f}s (strategy=scan)")
+            finally:
+                # explicitly close the results generator to prevent connection leaks
+                if results_generator is not None:
+                    results_generator.close()
 
     def is_column_indexed(self, column: str) -> bool:
         with self.container.use_sa_engine() as sa_engine:
-            indexes = sa.inspect(sa_engine).get_indexes(table_name=self.name, schema=self.container.dbschema)
+            indexes = sa.inspect(sa_engine).get_indexes(table_name=self.db_table_name, schema=self.container.dbschema)
         for index in indexes:
             if column in index.get("column_names", []):
                 return True
@@ -1214,7 +1229,7 @@ class SqlAlchemyTable(DBTable, abc.ABC):
         do_coerce_dtypes: bool | None = False,
     ) -> pd.DataFrame:
         t0 = time.time()
-        _LOG.info(f"read from table `{self.name}` started")
+        _LOG.info(f"read from table `{self.db_table_name}` started")
         columns = columns if columns is not None else self.columns
         stmt = self._sa_select(columns)
         stmts = self._sa_where(stmt, where)
@@ -1232,7 +1247,7 @@ class SqlAlchemyTable(DBTable, abc.ABC):
             df = self._df_limit(df, limit)
         if do_coerce_dtypes:
             df = coerce_dtypes_by_encoding(df, self.encoding_types)
-        _LOG.info(f"read DB data `{self.name}` {df.shape} in {time.time() - t0:.2f}s")
+        _LOG.info(f"read DB data `{self.db_table_name}` {df.shape} in {time.time() - t0:.2f}s")
         return df
 
     def create_table(self, df: pd.DataFrame | None = None, **kwargs) -> None:
@@ -1245,7 +1260,7 @@ class SqlAlchemyTable(DBTable, abc.ABC):
         with self.container.use_sa_engine(mode="write_data") as sa_engine:
             try:
                 df.head(0).to_sql(
-                    name=self.name,
+                    name=self.db_table_name,
                     con=sa_engine,
                     schema=self.container.dbschema,
                     index=False,
@@ -1257,7 +1272,7 @@ class SqlAlchemyTable(DBTable, abc.ABC):
                 raise
 
         dtypes_msg = f"with dtypes=`{kwargs['dtype']}`" if "dtype" in kwargs else ""
-        _LOG.info(f"Successfully created table `{self.name}` schema {dtypes_msg}")
+        _LOG.info(f"Successfully created table `{self.db_table_name}` schema {dtypes_msg}")
 
     def write_chunks(self, chunks: Iterable[pd.DataFrame], dtypes: dict[str, Any], **kwargs) -> None:
         _LOG.info(f"write data in {len(chunks)} chunks (n_jobs={self.WRITE_CHUNKS_N_JOBS})")
@@ -1269,7 +1284,7 @@ class SqlAlchemyTable(DBTable, abc.ABC):
                     sa_create_engine_kwargs=self.container.sa_create_engine_kwargs,
                     sa_create_engine_connect_kwargs=self.container.sa_engine_connection_kwargs,
                     sa_multiple_inserts=self.SA_MULTIPLE_INSERTS,
-                    table_name=self.name,
+                    table_name=self.db_table_name,
                     table_schema=self.container.dbschema,
                     table_dtypes=dtypes,
                     chunk_init=self.INIT_WRITE_CHUNK,
@@ -1289,7 +1304,7 @@ class SqlAlchemyTable(DBTable, abc.ABC):
     ) -> None:
         t0 = time.time()
         assert self.is_output
-        _LOG.info(f"write data {df.shape} to table `{self.name}` started")
+        _LOG.info(f"write data {df.shape} to table `{self.db_table_name}` started")
 
         df = _normalize_for_sql(df)
 
@@ -1306,7 +1321,7 @@ class SqlAlchemyTable(DBTable, abc.ABC):
         chunks = [df[i : i + write_chunk_size] for i in range(0, len(df), write_chunk_size)]
         self.write_chunks(chunks, dtypes)
 
-        _LOG.info(f"write to table `{self.name}` finished in {time.time() - t0:.2f}s")
+        _LOG.info(f"write to table `{self.db_table_name}` finished in {time.time() - t0:.2f}s")
 
     def has_child(self) -> bool:
         return len(self.container.schema.get_relations_from_table(str(self.name))) > 0
