@@ -282,7 +282,7 @@ class DatabricksTable(SqlAlchemyTable):
         if_exists = kwargs.get("if_exists", "append")
         with self.container.init_sa_connection() as connection:
             cursor = connection.connection.cursor()
-            cursor.columns(schema_name=self.container.dbschema, table_name=self.name)
+            cursor.columns(schema_name=self.container.dbschema, table_name=self.db_table_name)
             result = cursor.fetchall()
             if result:  # the table already exists
                 if if_exists == "replace":
@@ -291,9 +291,9 @@ class DatabricksTable(SqlAlchemyTable):
                 elif if_exists == "fail":
                     raise MostlyDataException("Destination location already exists.")
             # create an empty table without schema
-            query = f"CREATE TABLE IF NOT EXISTS {self.container.dbname}.{self.container.dbschema}.{self.name}"
+            query = f"CREATE TABLE IF NOT EXISTS {self.container.dbname}.{self.container.dbschema}.{self.db_table_name}"
             cursor.execute(query)
-            _LOG.info(f"created table `{self.name}` under schema `{self.container.dbschema}`")
+            _LOG.info(f"created table `{self.db_table_name}` under schema `{self.container.dbschema}`")
 
     def create_volume(self, volume_name) -> None:
         self.drop_volume_if_exists(volume_name)
@@ -307,20 +307,20 @@ class DatabricksTable(SqlAlchemyTable):
         _LOG.info(f"dropped temporary volume `{volume_name}` if existed")
 
     def drop_table_if_exists(self) -> None:
-        query = f"DROP TABLE IF EXISTS {self.container.dbname}.{self.container.dbschema}.{self.name};"
+        query = f"DROP TABLE IF EXISTS {self.container.dbname}.{self.container.dbschema}.{self.db_table_name};"
         self._execute(query)
-        _LOG.info(f"dropped table `{self.name}` if existed")
+        _LOG.info(f"dropped table `{self.db_table_name}` if existed")
 
     def copy_data_from_volume_to_table(self, volume_name: str) -> None:
         query = (
-            f"COPY INTO {self.container.dbname}.{self.container.dbschema}.{self.name} "
+            f"COPY INTO {self.container.dbname}.{self.container.dbschema}.{self.db_table_name} "
             f"FROM '/Volumes/{self.container.dbname}/{self.container.dbschema}/{volume_name}/' "
             "FILEFORMAT = PARQUET "
             "FORMAT_OPTIONS ('inferSchema' = 'true') "
             "COPY_OPTIONS ('mergeSchema' = 'true');"
         )
         self._execute(query)
-        _LOG.info(f"copied data from volume `{volume_name}` to table `{self.name}`")
+        _LOG.info(f"copied data from volume `{volume_name}` to table `{self.db_table_name}`")
 
     def write_data(
         self,
@@ -330,7 +330,7 @@ class DatabricksTable(SqlAlchemyTable):
     ) -> None:
         t0 = time.time()
         assert self.is_output
-        _LOG.info(f"write data {df.shape} to table `{self.name}` started")
+        _LOG.info(f"write data {df.shape} to table `{self.db_table_name}` started")
         # include a unique timestamp in the volume name to avoid conflicts
         volume_name = f"{self.name}_temp_{time.time_ns()}"
         try:
@@ -340,7 +340,7 @@ class DatabricksTable(SqlAlchemyTable):
             self.write_data_to_volume(df=df, volume_name=volume_name)
             self.copy_data_from_volume_to_table(volume_name)
 
-            _LOG.info(f"write to table `{self.name}` finished in {time.time() - t0:.2f}s")
+            _LOG.info(f"write to table `{self.db_table_name}` finished in {time.time() - t0:.2f}s")
         except Exception as e:
             if if_exists != "append":
                 self.drop_table_if_exists()
