@@ -43,6 +43,7 @@ from mostlyai.sdk._local.execution.step_finalize_generation import (
     execute_step_finalize_generation,
     update_total_rows,
 )
+from mostlyai.sdk._local.execution.step_finalize_training import execute_step_finalize_training
 from mostlyai.sdk._local.execution.step_generate_data import execute_step_generate_data
 from mostlyai.sdk._local.execution.step_generate_model_report_data import (
     execute_step_generate_model_report_data,
@@ -85,18 +86,23 @@ def _set_random_state(random_state: int | None = None):
 
 
 def _move_training_artefacts(generator_dir: Path, job_workspace_dir: Path):
-    for dir in ["Logs", "ModelStore", "ModelQAReports", "ModelQAStatistics"]:
+    for dir in ["Logs", "ModelStore", "ModelQAReports", "ModelQAStatistics", "FKModelsStore"]:
         shutil.rmtree(generator_dir / dir, ignore_errors=True)
         (generator_dir / dir).mkdir()
     for path in job_workspace_dir.absolute().rglob("*"):
-        if path.is_dir() and path.name == "ModelStore":
-            model_label = path.parent.name
-            path.rename(generator_dir / "ModelStore" / model_label)
-        if path.is_file() and path.parent.name == "ModelQAReports":
-            path.rename(generator_dir / "ModelQAReports" / path.name)
-        if path.is_dir() and path.name == "ModelQAStatistics":
-            model_label = path.parent.name
-            path.rename(generator_dir / "ModelQAStatistics" / model_label)
+        if "FKModelsStore" not in path.parts:
+            if path.is_dir() and path.name == "ModelStore":
+                model_label = path.parent.name
+                path.rename(generator_dir / "ModelStore" / model_label)
+            if path.is_dir() and path.name == "FKModelsStore":
+                path.rename(generator_dir / "FKModelsStore")
+            if path.is_file() and path.parent.name == "ModelQAReports":
+                path.rename(generator_dir / "ModelQAReports" / path.name)
+            if path.is_dir() and path.name == "ModelQAStatistics":
+                model_label = path.parent.name
+                path.rename(generator_dir / "ModelQAStatistics" / model_label)
+        if path.is_dir() and path.name == "FKModelsStore":
+            path.rename(generator_dir / "FKModelsStore")
 
 
 def _move_generation_artefacts(synthetic_dataset_dir: Path, job_workspace_dir: Path):
@@ -180,6 +186,14 @@ def _copy_model(generator_dir: Path, model_label: str, workspace_dir: Path) -> b
     return False
 
 
+def _copy_fk_models(generator_dir: Path, job_workspace_dir: Path) -> bool:
+    models_path = generator_dir / "FKModelsStore"
+    if models_path.exists():
+        shutil.copytree(models_path, job_workspace_dir / "FKModelsStore")
+        return True
+    return False
+
+
 def _copy_statistics(generator_dir: Path, model_label: str, workspace_dir: Path) -> bool:
     statistics_path = generator_dir / "ModelQAStatistics" / model_label
     if statistics_path.exists():
@@ -431,7 +445,19 @@ class Execution:
                 tgt_table.language_model_metrics = model_metrics
 
     def execute_task_finalize_training(self, task: Task):
-        pass
+        generator = self._generator
+        generator_dir = self._home_dir / "generators" / generator.id
+        connectors = [
+            read_connector_from_json(self._home_dir / "connectors" / t.source_connector_id) for t in generator.tables
+        ]
+        execute_step_finalize_training(
+            generator=generator,
+            connectors=connectors,
+            job_workspace_dir=self._job_workspace_dir,
+            update_progress=LocalProgressCallback(
+                resource_path=generator_dir, model_label=None, step_code=StepCode.finalize_training
+            ),
+        )
 
     def execute_task_generate(self, task: Task):
         generator = self._generator
@@ -531,16 +557,23 @@ class Execution:
                 self.execute_deliver_data()
 
     def execute_finalize_generation(self):
+        generator = self._generator
+        generator_dir = self._home_dir / "generators" / generator.id
+        job_workspace_dir = self._job_workspace_dir
+
+        # copy FK models to job workspace
+        _copy_fk_models(generator_dir=generator_dir, job_workspace_dir=job_workspace_dir)
+
         schema = create_generation_schema(
             generator=self._generator,
-            job_workspace_dir=self._job_workspace_dir,
+            job_workspace_dir=job_workspace_dir,
             step="finalize_generation",
         )
 
         usages = execute_step_finalize_generation(
             schema=schema,
             is_probe=False,
-            job_workspace_dir=self._job_workspace_dir,
+            job_workspace_dir=job_workspace_dir,
             update_progress=LocalProgressCallback(
                 resource_path=self._home_dir / "synthetic-datasets" / self._synthetic_dataset.id,
                 model_label=None,
@@ -572,16 +605,23 @@ class Execution:
         )
 
     def execute_finalize_probing(self):
+        generator = self._generator
+        generator_dir = self._home_dir / "generators" / generator.id
+        job_workspace_dir = self._job_workspace_dir
+
+        # copy FK models to job workspace
+        _copy_fk_models(generator_dir=generator_dir, job_workspace_dir=job_workspace_dir)
+
         schema = create_generation_schema(
             generator=self._generator,
-            job_workspace_dir=self._job_workspace_dir,
+            job_workspace_dir=job_workspace_dir,
             step="finalize_generation",
         )
         # step: FINALIZE_GENERATION
         _ = execute_step_finalize_generation(
             schema=schema,
             is_probe=True,
-            job_workspace_dir=self._job_workspace_dir,
+            job_workspace_dir=job_workspace_dir,
         )
 
 
