@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import logging
+import math
 import uuid
 import zipfile
 from pathlib import Path
@@ -24,7 +25,14 @@ from mostlyai.sdk._data.dtype import is_timestamp_dtype
 from mostlyai.sdk._data.file.base import LocalFileContainer
 from mostlyai.sdk._data.file.table.csv import CsvDataTable
 from mostlyai.sdk._data.file.table.parquet import ParquetDataTable
-from mostlyai.sdk._data.non_context import postproc_non_context
+from mostlyai.sdk._data.non_context import (
+    CHILDREN_COUNT_COLUMN_NAME,
+    add_context_parent_data,
+    assign_non_context_fks_randomly,
+    initialize_remaining_capacity,
+    match_non_context,
+    safe_name,
+)
 from mostlyai.sdk._data.progress_callback import ProgressCallback, ProgressCallbackWrapper
 from mostlyai.sdk._data.util.common import (
     IS_NULL,
@@ -35,6 +43,11 @@ from mostlyai.sdk.domain import Generator, ModelType, SyntheticDataset
 
 _LOG = logging.getLogger(__name__)
 
+# FK processing constants
+FK_MIN_CHILDREN_BATCH_SIZE = 10
+FK_MAX_CHILDREN_BATCH_SIZE = 10_000
+FK_PARENT_BATCH_SIZE = 1_000
+
 
 def execute_step_finalize_generation(
     *,
@@ -56,6 +69,7 @@ def execute_step_finalize_generation(
                 target_table_name=table_name,
                 delivery_dir=delivery_dir,
                 export_csv=False,
+                job_workspace_dir=job_workspace_dir,
             )
         return usages
 
@@ -76,6 +90,7 @@ def execute_step_finalize_generation(
                 target_table_name=tgt,
                 delivery_dir=delivery_dir,
                 export_csv=export_csv,
+                job_workspace_dir=job_workspace_dir,
             )
             progress.update(advance=1)
 
@@ -193,14 +208,263 @@ def export_random_samples(
         )
 
 
-def finalize_probing(schema: Schema, delivery_dir: Path):
-    for tgt in schema.tables:
-        finalize_table_generation(
+def filter_and_order_columns(data: pd.DataFrame, table_name: str, schema: Schema) -> pd.DataFrame:
+    """Keep only original columns in the right order."""
+    tgt_cols = schema.tables[table_name].columns or data.columns
+    drop_cols = [c for c in tgt_cols if c not in data]
+    if drop_cols:
+        _LOG.info(f"remove columns from final output: {', '.join(drop_cols)}")
+    keep_cols = [c for c in tgt_cols if c in data]
+    return data[keep_cols]
+
+
+def write_batch_outputs(
+    data: pd.DataFrame, table_name: str, batch_counter: int, pqt_path: Path, csv_path: Path | None
+) -> None:
+    """Write batch to parquet and optionally CSV."""
+    batch_filename = f"part.{batch_counter:06d}.parquet"
+    _LOG.info(f"store post-processed batch {batch_counter} ({len(data)} rows) as PQT")
+    pqt_post = ParquetDataTable(path=pqt_path / batch_filename, name=table_name)
+    pqt_post.write_data(data)
+
+    if csv_path:
+        _LOG.info(f"store post-processed batch {batch_counter} as CSV")
+        csv_post = CsvDataTable(path=csv_path / f"{table_name}.csv", name=table_name)
+        csv_post.write_data(data, if_exists="append")
+
+
+def setup_output_paths(delivery_dir: Path, target_table_name: str, export_csv: bool) -> tuple[Path, Path | None]:
+    """Create output directories for parquet and optionally CSV."""
+    pqt_path = delivery_dir / target_table_name / "parquet"
+    pqt_path.mkdir(exist_ok=True, parents=True)
+    _LOG.info(f"prepared {pqt_path=} for storing post-processed data as PQT files")
+
+    csv_path = None
+    if export_csv:
+        csv_path = delivery_dir / target_table_name / "csv"
+        csv_path.mkdir(exist_ok=True, parents=True)
+        _LOG.info(f"prepared {csv_path=} for storing post-processed data as CSV file")
+
+    return pqt_path, csv_path
+
+
+def are_fk_models_available(job_workspace_dir: Path, target_table_name: str, schema: Schema) -> bool:
+    """
+    Check if both FK models and cardinality models are available for the given table.
+    Returns True only if ALL required models exist for ALL non-context relations.
+    Falls back to random assignment if any model is missing.
+    """
+    fk_models_dir = job_workspace_dir / "FKModelsStore" / target_table_name
+
+    # Check if directory exists
+    if not fk_models_dir.exists():
+        return False
+
+    # Get all non-context relations for this table
+    non_ctx_relations = [rel for rel in schema.non_context_relations if rel.child.table == target_table_name]
+
+    # If no non-context relations, no models needed
+    if not non_ctx_relations:
+        return True
+
+    # Check that each relation has both FK model and cardinality model
+    for relation in non_ctx_relations:
+        fk_model_dir = fk_models_dir / safe_name(relation.child.column)
+
+        # Check FK model exists
+        matching_model_dir = fk_model_dir / "fk_matching_model"
+        fk_model_weights = matching_model_dir / "model_weights.pt"
+        if not fk_model_weights.exists():
+            _LOG.info(
+                f"FK model not found for {target_table_name}.{relation.child.column} "
+                f"at {fk_model_weights} - falling back to random assignment"
+            )
+            return False
+
+        # Check cardinality model exists
+        cardinality_model_dir = fk_model_dir / "cardinality_model"
+        cardinality_model_store = cardinality_model_dir / "ModelStore"
+        if not (cardinality_model_dir.exists() and cardinality_model_store.exists()):
+            _LOG.info(
+                f"Cardinality model not found for {target_table_name}.{relation.child.column} "
+                f"at {cardinality_model_dir} - falling back to random assignment"
+            )
+            return False
+
+    return True
+
+
+def process_table_with_random_fk_assignment(
+    table_name: str,
+    schema: Schema,
+    pqt_path: Path,
+    csv_path: Path | None,
+) -> None:
+    """Process table with random FK assignment, chunk by chunk."""
+    table = schema.tables[table_name]
+
+    for chunk_idx, chunk_data in enumerate(table.read_chunks(do_coerce_dtypes=True)):
+        _LOG.info(f"Processing chunk {chunk_idx + 1} ({len(chunk_data)} rows)")
+        processed_data = assign_non_context_fks_randomly(
+            tgt_data=chunk_data,
             generated_data_schema=schema,
-            target_table_name=tgt,
-            delivery_dir=delivery_dir,
-            export_csv=False,
+            tgt=table_name,
         )
+        processed_data = filter_and_order_columns(processed_data, table_name, schema)
+        write_batch_outputs(processed_data, table_name, chunk_idx, pqt_path, csv_path)
+
+
+def calculate_optimal_child_batch_size_for_relation(
+    parent_key_count: int,
+    children_row_count: int,
+    parent_batch_size: int,
+    relation_name: str,
+) -> int:
+    """Calculate optimal child batch size for a specific FK relationship."""
+    num_parent_batches = max(1, math.ceil(parent_key_count / parent_batch_size))
+
+    # ideal batch size for full parent utilization
+    ideal_batch_size = children_row_count // num_parent_batches
+
+    # ensure optimal batch size stays within defined min and max bounds
+    optimal_batch_size = min(max(ideal_batch_size, FK_MIN_CHILDREN_BATCH_SIZE), FK_MAX_CHILDREN_BATCH_SIZE)
+
+    # log utilization metrics
+    num_child_batches = children_row_count // optimal_batch_size
+    parent_utilization = min(num_child_batches / num_parent_batches * 100, 100)
+
+    _LOG.info(
+        f"[{relation_name}] Batch size optimization | "
+        f"total_children: {children_row_count} | "
+        f"parent_size: {parent_key_count} | "
+        f"parent_batch_size: {parent_batch_size} | "
+        f"parent_batches: {num_parent_batches} | "
+        f"ideal_child_batch: {ideal_batch_size} | "
+        f"optimal_child_batch: {optimal_batch_size} | "
+        f"parent_utilization: {parent_utilization:.1f}%"
+    )
+
+    return optimal_batch_size
+
+
+def process_table_with_fk_models(
+    *,
+    table_name: str,
+    schema: Schema,
+    pqt_path: Path,
+    csv_path: Path | None,
+    parent_batch_size: int = FK_PARENT_BATCH_SIZE,
+    job_workspace_dir: Path,
+) -> None:
+    """Process table with ML model-based FK assignment using logical child batches."""
+
+    fk_models_workspace_dir = job_workspace_dir / "FKModelsStore" / table_name
+    non_ctx_relations = [rel for rel in schema.non_context_relations if rel.child.table == table_name]
+    children_table = schema.tables[table_name]
+
+    # Load parent keys upfront (memory efficient)
+    parent_keys_cache = {}
+    parent_tables = {}
+    for relation in non_ctx_relations:
+        parent_table_name = relation.parent.table
+        if parent_table_name not in parent_keys_cache:
+            parent_table = schema.tables[parent_table_name]
+            parent_tables[parent_table_name] = parent_table
+            pk_col = relation.parent.column
+            parent_keys_cache[parent_table_name] = parent_table.read_data(
+                columns=[pk_col],
+                do_coerce_dtypes=True,
+            )
+
+    # Calculate optimal batch size for each relationship
+    relation_batch_sizes = {}
+    for relation in non_ctx_relations:
+        parent_table_name = relation.parent.table
+        parent_key_count = len(parent_keys_cache[parent_table_name])
+        relation_name = f"{relation.child.table}.{relation.child.column}->{parent_table_name}"
+
+        optimal_batch_size = calculate_optimal_child_batch_size_for_relation(
+            parent_key_count=parent_key_count,
+            children_row_count=children_table.row_count,
+            parent_batch_size=parent_batch_size,
+            relation_name=relation_name,
+        )
+        relation_batch_sizes[relation] = optimal_batch_size
+
+    remaining_capacity = {}
+    children_counts = {}
+    for relation in non_ctx_relations:
+        parent_table_name = relation.parent.table
+        parent_table = parent_tables[parent_table_name]
+        pk_col = relation.parent.column
+        fk_model_dir = fk_models_workspace_dir / safe_name(relation.child.column)
+
+        _LOG.info(f"Using Cardinality Model for {relation.child.table}.{relation.child.column}")
+        capacity_dict = initialize_remaining_capacity(
+            fk_model_workspace_dir=fk_model_dir,
+            parent_table=parent_table,
+            parent_pk=pk_col,
+        )
+        remaining_capacity[relation] = capacity_dict
+        children_counts[relation] = capacity_dict.copy()
+
+    for chunk_idx, chunk_data in enumerate(children_table.read_chunks(do_coerce_dtypes=True)):
+        _LOG.info(f"Processing chunk {chunk_idx} ({len(chunk_data)} rows)")
+
+        for relation in non_ctx_relations:
+            parent_table_name = relation.parent.table
+            parent_table = parent_tables[parent_table_name]
+            parent_pk = relation.parent.column
+            optimal_batch_size = relation_batch_sizes[relation]
+            relation_name = f"{relation.child.table}.{relation.child.column}->{parent_table_name}"
+
+            _LOG.info(f"  Processing relationship {relation_name} with batch size {optimal_batch_size}")
+
+            parent_keys_df = parent_keys_cache[parent_table_name]
+
+            processed_batches = []
+
+            for batch_start in range(0, len(chunk_data), optimal_batch_size):
+                batch_end = min(batch_start + optimal_batch_size, len(chunk_data))
+                batch_data = chunk_data.iloc[batch_start:batch_end].copy()
+
+                sampled_parent_keys = parent_keys_df.sample(
+                    n=parent_batch_size, replace=len(parent_keys_df) < parent_batch_size
+                )[parent_pk].tolist()
+
+                parent_data = parent_table.read_data(
+                    where={parent_pk: sampled_parent_keys},
+                    columns=parent_table.columns,
+                    do_coerce_dtypes=True,
+                )
+
+                parent_data[CHILDREN_COUNT_COLUMN_NAME] = (
+                    parent_data[parent_pk].map(children_counts[relation]).fillna(0).astype(int)
+                )
+
+                batch_data = add_context_parent_data(
+                    tgt_data=batch_data,
+                    tgt_table=children_table,
+                    schema=schema,
+                )
+
+                assert relation in remaining_capacity
+                processed_batch = match_non_context(
+                    fk_models_workspace_dir=fk_models_workspace_dir,
+                    tgt_data=batch_data,
+                    parent_data=parent_data,
+                    tgt_parent_key=relation.child.column,
+                    parent_primary_key=relation.parent.column,
+                    parent_table_name=parent_table_name,
+                    remaining_capacity=remaining_capacity[relation],
+                )
+
+                processed_batches.append(processed_batch)
+
+            chunk_data = pd.concat(processed_batches, ignore_index=True)
+
+        chunk_data = filter_and_order_columns(chunk_data, table_name, schema)
+        write_batch_outputs(chunk_data, table_name, chunk_idx, pqt_path, csv_path)
 
 
 def finalize_table_generation(
@@ -208,74 +472,47 @@ def finalize_table_generation(
     target_table_name: str,
     delivery_dir: Path,
     export_csv: bool,
+    job_workspace_dir: Path,
 ) -> None:
     """
     Post-process the generated data for a given table.
-    * handle non-context keys
+    * handle non-context keys (using FK models if available)
     * handle reference keys
     * keep only needed columns, and in the right order
     * export to PARQUET, and optionally also to CSV (without col prefixes)
     """
 
-    # read generated "raw" data into memory
-    # Note: We should avoid reading all data into memory. E.g. by looping over source.dataset.files or over
-    # source.dataset.to_batches(), then process in batches, and then append to CSV, respectively write out
-    # as separate parquet files.
-    table = generated_data_schema.tables[target_table_name]
-    tgt_table_files = table.dataset.files
-    n_partitions = len(tgt_table_files)
-    _LOG.info(f"POSTPROC will handle {n_partitions} partitions")
+    pqt_path, csv_path = setup_output_paths(delivery_dir, target_table_name, export_csv)
+    fk_models_available = are_fk_models_available(job_workspace_dir, target_table_name, generated_data_schema)
 
-    pqt_path = delivery_dir / target_table_name / "parquet"
-    pqt_path.mkdir(exist_ok=True, parents=True)
-    _LOG.info(f"prepared {pqt_path=} for storing post-processed data as PQT files")
-
-    if export_csv:
-        csv_path = delivery_dir / target_table_name / "csv"
-        csv_path.mkdir(exist_ok=True, parents=True)
-        _LOG.info(f"prepared {csv_path=} for storing post-processed data as CSV file")
+    if fk_models_available:
+        _LOG.info(f"Assigning non context FKs (if exists) through FK models for table {target_table_name}")
+        try:
+            process_table_with_fk_models(
+                table_name=target_table_name,
+                schema=generated_data_schema,
+                pqt_path=pqt_path,
+                csv_path=csv_path,
+                job_workspace_dir=job_workspace_dir,
+            )
+        except Exception as e:
+            _LOG.error(f"FK model processing failed for table {target_table_name}: {e}")
+            _LOG.warning(f"Falling back to random assignment for table {target_table_name}")
+            process_table_with_random_fk_assignment(
+                table_name=target_table_name,
+                schema=generated_data_schema,
+                pqt_path=pqt_path,
+                csv_path=csv_path,
+            )
     else:
-        csv_path = None
-
-    # instantiate container outside of loop to avoid memory to pile up
-    container = type(table.container)()
-    for part_i, part_file in enumerate(tgt_table_files, start=1):
-        container.set_location(part_file)
-        part_table = ParquetDataTable(container=container)
-        tgt_data = part_table.read_data(do_coerce_dtypes=True)
-
-        # post-process non-context keys
-        tgt_data = postproc_non_context(
-            tgt_data=tgt_data,
-            generated_data_schema=generated_data_schema,
-            tgt=target_table_name,
+        _LOG.info(f"Assigning non context FKs (if exists) through random assignment for table {target_table_name}")
+        process_table_with_random_fk_assignment(
+            table_name=target_table_name,
+            schema=generated_data_schema,
+            pqt_path=pqt_path,
+            csv_path=csv_path,
         )
 
-        # keep only original columns, and in the right order
-        tgt_cols = (
-            cols if (cols := generated_data_schema.tables[target_table_name].columns) is not None else tgt_data.columns
-        )
-        drop_cols = [c for c in tgt_cols if c not in tgt_data]
-        if drop_cols:
-            _LOG.info(f"remove columns from final output: {', '.join(drop_cols)}")
-        keep_cols = [c for c in tgt_cols if c in tgt_data]
-        tgt_data = tgt_data[keep_cols]
-
-        partition_text = f"{part_i} out of {n_partitions}"
-
-        # store post-processed data as PQT files
-        _LOG.info(f"store post-processed {partition_text} as PQT")
-        pqt_post = ParquetDataTable(path=pqt_path / f"{Path(part_file).stem}.parquet", name=target_table_name)
-        pqt_post.write_data(tgt_data)
-
-        # store post-processed data as single CSV file
-        if csv_path:
-            _LOG.info(f"store post-processed {partition_text} as CSV")
-            csv_post = CsvDataTable(path=csv_path / f"{target_table_name}.csv", name=target_table_name)
-            csv_post.write_data(tgt_data)
-
-        del tgt_data
-
 
 def export_data_to_excel(delivery_dir: Path, output_dir: Path):
     output_dir.mkdir(parents=True, exist_ok=True)
