@@ -567,6 +567,47 @@ class MostlyAI(_MostlyBaseClient):
             }, start=True, wait=True)
             ```
 
+        Example of a multi-table relational dataset with non-context foreign key:
+            ```python
+            # read original data
+            import pandas as pd
+            repo_url = 'https://github.com/mostly-ai/public-demo-data/raw/refs/heads/dev/berka/data'
+            accounts_df = pd.read_csv(f'{repo_url}/account.csv.gz')
+            disp_df = pd.read_csv(f'{repo_url}/disp.csv.gz')
+            clients_df = pd.read_csv(f'{repo_url}/client.csv.gz')
+            # instantiate client
+            from mostlyai.sdk import MostlyAI
+            mostly = MostlyAI()
+            # train generator
+            g = mostly.train(config={
+                'name': 'BERKA',
+                'tables': [{
+                    'name': 'clients',
+                    'data': clients_df,
+                    'primary_key': 'client_id',       # define PK column
+                }, {
+                    'name': 'accounts',
+                    'data': accounts_df,
+                    'primary_key': 'account_id',      # define PK column
+                }, {
+                    'name': 'disp',
+                    'data': disp_df,
+                    'primary_key': 'disp_id',         # define PK column
+                    'foreign_keys': [{                # define FK columns: max 1 Context FK allowed; referenced context tables must NOT result in circular references;
+                        'column': 'client_id',
+                        'referenced_table': 'clients',
+                        'is_context': True            # Context FK: the `disp` records that belong to the same `client` will be learned and generated together - with the context of the parent;
+                                                      # -> patterns between child and parent (and grand-parent) and between siblings belonging to the same parent will all be retained;
+                    }, {
+                        'column': 'account_id',
+                        'referenced_table': 'accounts',
+                        'is_context': False           # Non-Context FK: a dedicated model will be trained to learn matching a `disp` record with a suitable `account` record;
+                                                      # -> patterns between child and parent will be retained, but not between siblings belonging to the same parent;
+                    }],
+                }],
+            }, start=True, wait=True)
+            ```
+
         Example of a single flat table with TABULAR and LANGUAGE models:
             ```python
             # read original data
@@ -658,7 +699,7 @@ class MostlyAI(_MostlyBaseClient):
             generator (Generator | str): The generator instance or its UUID.
             config (SyntheticDatasetConfig | dict | None): Configuration for the synthetic dataset.
             size (int | dict[str, int] | None): Sample size(s) for the subject table(s).
-            seed (Seed | dict[str, Seed] | None): Seed data for the subject table(s).
+            seed (Seed | dict[str, Seed] | None): Either a single Seed for the subject table, or a dictionary with table names as keys and Seeds as values. Seed can either be a DataFrame or a path to a CSV or PARQUET file. Check generator details (`generator.tables[i].columns[j].value_range`) for possible value ranges.
             name (str | None): Name of the synthetic dataset.
             start (bool): Whether to start generation immediately. Default is True.
             wait (bool): Whether to wait for generation to finish. Default is True.
@@ -738,7 +779,7 @@ class MostlyAI(_MostlyBaseClient):
         Args:
             generator (Generator | str): The generator instance or its UUID.
             size (int | dict[str, int] | None): Sample size(s) for the subject table(s). Default is 1, if no seed is provided.
-            seed (Seed | dict[str, Seed] | None): Seed data for the subject table(s). Check generator details for possible value ranges.
+            seed (Seed | dict[str, Seed] | None): Either a single Seed for the subject table, or a dictionary with table names as keys and Seeds as values. Seed can either be a DataFrame or a path to a CSV or PARQUET file. Check generator details (`generator.tables[i].columns[j].value_range`) for possible value ranges. Seed data may contain null values; columns listed in `imputation.columns` will have nulls generated, while unlisted columns keep nulls as-is.
             config (SyntheticProbeConfig | dict | None): Configuration for the probe.
             return_type (Literal["auto", "dict"]): The type of the return value. "dict" will always provide a dictionary of DataFrames. "auto" will return a single DataFrame for a single-table generator, and a dictionary of DataFrames for a multi-table generator. Default is "auto".
 
@@ -766,8 +807,11 @@ class MostlyAI(_MostlyBaseClient):
             import pandas as pd
             from mostlyai.sdk import MostlyAI
             mostly = MostlyAI()
+            # seed with nulls for columns to be imputed
+            seed = pd.DataFrame({'country': ['US', 'CA'], 'age': [None, None], 'income': [50000, None]})
             data = mostly.probe(
                 'INSERT_YOUR_GENERATOR_ID',
+                seed=seed,
                 config={
                     'tables': [{
                         'name': 'tbl1',
@@ -775,8 +819,8 @@ class MostlyAI(_MostlyBaseClient):
                             'sample_size': 100,
                             'sampling_temperature': 1.0,
                             'sampling_top_p': 1.0,
-                            'rebalancing': {'column': 'country', 'probabilities': {'US': 0.5, 'CA', 0.3}},
-                            'imputation': {'columns': ['age']},
+                            'rebalancing': {'column': 'country', 'probabilities': {'US': 0.5, 'CA': 0.3}},
+                            'imputation': {'columns': ['age']},  # impute age nulls; income null stays as-is
                             'fairness': {'target_column': 'income', 'sensitive_columns': ['gender']},
                         }
                     }]
@@ -784,6 +828,24 @@ class MostlyAI(_MostlyBaseClient):
                 return_type="dict"
             )
             ```
+
+        Example for multi-table conditional probing (e.g., time-series with 100 simulations):
+            ```python
+            # create 100 simulations for a specific user profile and first 2 purchases
+            user_ids = [f"sim-{i:03d}" for i in range(100)]
+            seed_users = pd.DataFrame({'users_id': user_ids})
+            seed_purchases = pd.DataFrame({
+                'users_id': [uid for uid in user_ids for _ in range(2)],
+                'date': pd.to_datetime(['1997-01-12', '1997-01-12'] * 100),
+                'cds': [1, 5] * 100,
+                'amt': [12.00, 77.00] * 100,
+            })
+            data = mostly.probe(
+                'INSERT_YOUR_GENERATOR_ID',
+                seed={'users': seed_users, 'purchases': seed_purchases}
+            )
+            # Note: For multi-table seeds, provide unique PK/FK values to match records between tables
+            ```
         """
         config = harmonize_sd_config(
             generator,
